#ifndef __FPGA_PACKET_H__
#define __FPGA_PACKET_H__

// ****************************************************************************



/* Extern references to the generated ESB packets and count */
extern uint16_t esb_payload_count;      /* How many ESB payloads actually got filled */
extern const char * Fpga_put_rslt_txt[];

//==============================================================================
// LIB

// OS
#include <zephyr/types.h>
#include <esb.h>

// APP
// -
//==============================================================================
// *****************************************************************************
// *****************************************************************************
//                          Test management directives
#define TEST_FPGA_PRINT_PACKETS     0       /* enable/disable - print of FPGA packet */
#define TEST_FPGA_PRINT_DEBUG       0       /* enable/disable - print the debug info from ESB creating */
#define TEST_FPGA_PUT_PRINT_STATE   0       /* enable/disable - print the state variables debug info from fpga_put_esb2fpga() */
#define TEST_FPGA_PUT_PRINT_ESB     0       /* enable/disable - print the ESB debug info from fpga_put_esb2fpga() */
#define TEST_FPGA_PUT_PRINT_FPGA    0       /* enable/disable - print the FPGA debug info from fpga_put_esb2fpga() */
#define TEST_FPGA_PUT_CHECK_CRC     1       /* enable/disable - CRC chceck in fpga_put_esb2fpga() */
//==============================================================================
// MS #define CONFIG_ESB_MAX_PAYLOAD_LENGTH 252  // ESB payload limit
#define FPGA_PACKET_NUM 10U                                 // selected by user
#define ESB_PACKET_PER_FPGA_PACKET 5                        // number of ESB packets per one FPGA packet: MAX_PACKET_SIZE / sizeof(esb_payload) = 1040 / 251 = 4 rest. 36 ---> 5
#define ESB_PACKET_NUM (FPGA_PACKET_NUM * ESB_PACKET_PER_FPGA_PACKET)        // ESB packet buffer for all FPGA packets
#define ESB_SUBIDX_NOINIT (-1)                              // no initialized esb subindex
#define ESB_SUBIDX_FIRST 0                                  // first subindex in FPGA package
#define ESB_SUBIDX_LAST (ESB_PACKET_PER_FPGA_PACKET - 1)    // last subindex in FPGA package

#define FPGA_PACKET_SIZE (sizeof(FPacket_t))
#define FPGA_CRC_SEED   (0x0000)                            // (0) seed for CRC calculation for FPGA packet
#define FPGA_CRC_VALUE_SIZE   (sizeof(uint16_t))            // number of bytes for CRC value
#define FPGA_CRC_SRC_SIZE   (FPGA_PACKET_SIZE - FPGA_CRC_VALUE_SIZE)        // number of bytes to calculate CRC

#define FPGA_HEADER_SIZE (sizeof(uint16_t))                 // FPacket_t.headerID
#define FPGA_ESBIDX_SIZE (sizeof(uint8_t))                  // ESB index size in PFGA package
#define FPGA_INFO_SIZE (FPGA_ESBIDX_SIZE + FPGA_HEADER_SIZE)    // size of ESB info (header and index)

#define BATCH_FPGA_HEADER_IDX (0)                             // index of FPGA header ID in the FPGA packet divided into ESB packets (batch)
#define BATCH_ESB_SUBIND_IDX (FPGA_HEADER_SIZE)               // index for ESB subindex within FPGA packet divided into ESB packets (bath)
#define BATCH_DATA_IDX (BATCH_ESB_SUBIND_IDX + FPGA_ESBIDX_SIZE) // index for ESB subindex within FPGA packet divided into ESB packets (batch)
//==============================================================================


// #############################################################################
//              				Makros and definitions
// #############################################################################


// *****************************************************************************
// 						Defining enumerators and structures
// *****************************************************************************
/** @brief Fpga packet result states. */
typedef enum  {
    PART_FIRST_OK = 0,                  /* first part (esb) of the FPGA was received correctly */
    PART_LAST_OK,                       /* first part (esb) of the FPGA was received correctly */
    PART_OK,                            /* another part (esb) of the FPGA was received correctly */
    PARTS_ALL_OK,                       /* all parts (esb) of the FPGA were received correctly (CRCreceived == CRCcalculated) */
    CRC_ERR,                            /* The calculated CRC of the received data is not the same as the received CRC. */
    FPGA_HEADER_ID_ERR,                 /* FPGA ID header error */
    ESB_IDX_ERR,                        /* ESB index error */

} FPGA_PUT_RESULT_e;
//==============================================================================
#define FPGA_CHAN_NUM   512U
#define FPGA_CHAN_SIZE  1U
#define FPGA_PAYLOAD_SIZE (FPGA_CHAN_NUM * FPGA_CHAN_SIZE)
// ****************************************************************************
typedef struct FPacket {
    uint16_t headerID;
    uint16_t timeStampMSB;
    uint16_t timeStampLSB;
    uint16_t measurementInfo;
    uint16_t payloadSize;
    uint16_t payloadID;
    uint16_t payload[FPGA_PAYLOAD_SIZE];
    uint16_t footerID;
    uint16_t CRC16;
} FPacket_t;
// *****************************************************************************
// 								Publishing global variables
// *****************************************************************************
// memory region:                   DATA
// *****************************************************************************

// *****************************************************************************
// memory region:            		CODE
// *****************************************************************************

// *****************************************************************************
//                             Function prototypes
// *****************************************************************************
void fill_f_packets(void);
uint8_t fill_esb_packets(struct esb_payload * esb_payload_ptr);
FPGA_PUT_RESULT_e fpga_put_esb2fpga(FPacket_t * fpga_ptr, struct esb_payload * esb_ptr);
void compare_packets(struct esb_payload *esb_payload_ptr, struct esb_payload *esb_pattern_ptr, uint16_t index);
void print_data_array(uint8_t * data_ptr, uint16_t data_size, uint8_t data_line_size);
void print_packet_esb_single(struct esb_payload * esb_payload_ptr, uint8_t esb_packet_id, bool print_data_enable);
void print_packet_esb_array(struct esb_payload * esb_payload_ptr, uint8_t esb_packet_num, bool print_data_enable);
uint16_t fpga_packet_crc_calc(FPacket_t * fpga_packet_ptr);


#endif /* __FPGA_PACKET_H__ */
